import { app } from 'electron';
import displayCrashScreen from './lib/crash';
import express from 'express';
import path from 'path';
import { createServer } from 'http';
import WebSocket from 'ws';
import { displayMainMenu } from './screens/mainmenu';
import { displayOptions } from './screens/options';
import { displayPlayMenu } from './screens/play';
import EventDispatcher from './lib/nfevents';
import { wdata } from './lib/windowManager';
import { screenSize, wait } from './lib/utils';

const eventDispatcher = new EventDispatcher();

declare global {
  namespace NodeJS {
    interface Process {
      nfenv?: any;
    }
  }
}

process.nfenv = {
  serverPort: 65034 as number, // todo: pick a random one instead, not for now bcuz easier to debug and use /eval endpoint
  crash: function (reason?: string) {
    // todo: disaply error in the crashScreen itself
    console.error('[nfcrash] NeoFunking crashed with reason:', reason || '*no reason*');
    displayCrashScreen();
  }
}

async function startWebRoot() {
  const app = express();
  const server = createServer(app);
  const wss = new WebSocket.Server({ server });

  app.use(express.static(path.join(__dirname, 'webroot')));

  app.get('/debug', (req: express.Request, res: express.Response): any => {
    console.log(screenSize());
    res.status(200).send('OK');
  });

  app.get('/eval', async (req: express.Request, res: express.Response): Promise<any> => {
    const code = req.headers['x-code'] as string;
  
    if (!code) return res.status(400).send('No code provided');
  
    try {
      const result = (async function(require) {
        return await eval(decodeURIComponent(code));
      })(require);
      res.status(200).send(await result as string);
    } catch (error: any) {
      res.status(500).send(error.message as string);
    }
  });

  wss.on('connection', (ws: WebSocket) => {
    console.log('[ws] client++');
    eventDispatcher.dispatch('nf_ws_connection', 'open');

    ws.on('message', (message) => {
      const msg = JSON.parse(message.toString());
      if (!msg.ch) msg.ch = 'public'
      ws.send(JSON.stringify(msg));
      console.log('[ws]', msg.ch, '-', msg.m);
      eventDispatcher.dispatch('nf_ws_message', msg);
    });

    ws.on('close', () => {
      console.log('[ws] client--');
      eventDispatcher.dispatch('nf_ws_connection', 'close');
    });
  });

  server.listen(process.nfenv.serverPort, 'localhost');
};

app.on('ready', async () => {
  try {
    startWebRoot();
    displayMainMenu();
    eventDispatcher.on('nf_ws_message', async (message) => {
      // screenManager
      if (message.m.startsWith('OPENSCREEN')) {
        const screen = message.m.split(':')[1];
        console.log('[screenManager] Trying to dispatch screen', screen)
        const bwdata = { ...wdata };
        switch (screen) {
          case 'Options':
            displayOptions();
            break;
          case 'Play':
            displayPlayMenu();
            break;
          case 'MainMenu':
            displayMainMenu();
            break;
          default:
            console.error('[screenManager] Unknown screen:', screen);
            displayCrashScreen();
        };
        await wait(200);
        Object.keys(bwdata).forEach(async (id) => {
          bwdata[id].win.destroy();
          delete bwdata[id]; // Cleanup
        });
        console.log('[screenManager] Cleaned old windows');
      }


    });
  } catch (e: any) {
    console.error(e);
    process.nfenv.crash(e.message);
  }
});