import { wm, wdata, WindowData } from './windowManager';
import { toP } from './utils'
import { BrowserWindow, screen } from 'electron';

export default async function displayCrashScreen() {
  const bwdata = { ...wdata };
  const { width: sw, height: sh } = screen.getPrimaryDisplay().bounds;
  const win = wm.create({
    w: 40,
    h: 40,
    whp: true,
    onCreate: (win: WindowData) => { win.win.loadURL(`http://localhost:${process.nfenv.serverPort}/screens/crash/index.html`) },
    onMaximize: (win: WindowData) => { win.win.restore() },
    onResize: (win: WindowData) => { win.win.setSize(win.conf.w, win.conf.h) },
    customProps: { maximizable: false }
  });
  const button = wm.create({
    w: Math.round(toP(sw, 10)),
    h: Math.round(toP(sh, 5)),
    noBorder: true,
    noBackground: true,
    onCreate: (win: WindowData) => { win.win.loadURL(`http://localhost:${process.nfenv.serverPort}/buttons/exit.html?code=1`) },
    onClose: () => { process.exit(1) },
    onMaximize: (win: WindowData) => { win.win.restore() },
    onResize: (win: WindowData) => { win.win.setSize(win.conf.w, win.conf.h) },
    customProps: { maximizable: false }
  });
  Object.keys(bwdata).forEach(async (id) => {
    bwdata[id].win.destroy(); // Note: We dont use wm.destroy(...) so ...onDestroy wont get called and cause trouble
    //                           Note 2: We also don't delete bwdata[id] because these windows cannot be recreated after a crash
    //                                     (The crash screen only allows the user to view and then exit the engine which will do the same result)
  });
  console.log('[nfcrash] Deleted all windows');
  const { height: csh, y: csy } = wdata[win].win.getBounds();
  wm.move({ id: button, y: csy + csh + toP(sh, 6), x: toP(sw, 50), fromCenter: true });
  wm.follow.start(button, win);
};