type Listener<T> = (data: T) => void;

export default class EventDispatcher<T = any> {
	private listeners: { [event: string]: Listener<T>[] } = {};

	on(event: string, listener: Listener<T>): void {
		if (!this.listeners[event]) {
			this.listeners[event] = [];
		}
		this.listeners[event].push(listener);
	}

	off(event: string, listener: Listener<T>): void {
		if (!this.listeners[event]) return;

		this.listeners[event] = this.listeners[event].filter(l => l !== listener);
	}

	dispatch(event: string, data: T): void {
		if (!this.listeners[event]) return;

		this.listeners[event].forEach(listener => {
			listener(data);
		});
	}
}
